@@ -62,6 +62,10 @@ point3f lambertian::get_albedo(const ray& r_in, const hit_record& rec) const {
   return(albedo->value(rec.u, rec.v, rec.p));
 }
 
+size_t lambertian::GetSize()  {
+  return(sizeof(*this));
+}
+
 //
 //Metal
 //
@@ -104,6 +108,10 @@ point3f metal::get_albedo(const ray& r_in, const hit_record& rec) const {
   return(albedo->value(rec.u, rec.v, rec.p));
 }
 
+size_t metal::GetSize()  {
+  return(sizeof(*this));
+}
+
 //
 //Dielectric
 //
@@ -312,6 +320,10 @@ bool dielectric::scatter(const ray& r_in, const hit_record& hrec, scatter_record
   return(true);
 }
 
+size_t dielectric::GetSize()  {
+  return(sizeof(*this));
+}
+
 //
 //Diffuse Light
 //
@@ -329,6 +341,10 @@ point3f diffuse_light::get_albedo(const ray& r_in, const hit_record& rec) const
   return(emit->value(rec.u, rec.v, rec.p));
 }
 
+size_t diffuse_light::GetSize()  {
+  return(sizeof(*this));
+}
+
 //
 //Spot Light
 //
@@ -358,6 +374,10 @@ point3f spot_light::get_albedo(const ray& r_in, const hit_record& rec) const {
   return(emit->value(rec.u, rec.v, rec.p));
 }
 
+size_t spot_light::GetSize()  {
+  return(sizeof(*this));
+}
+
 //
 //Isotropic
 //
@@ -382,6 +402,10 @@ point3f isotropic::get_albedo(const ray& r_in, const hit_record& rec) const {
   return(albedo->value(rec.u, rec.v, rec.p));
 }
 
+size_t isotropic::GetSize()  {
+  return(sizeof(*this));
+}
+
 //
 //Oren Nayar
 //
@@ -442,6 +466,10 @@ point3f orennayar::get_albedo(const ray& r_in, const hit_record& rec) const {
   return(albedo->value(rec.u, rec.v, rec.p));
 }
 
+size_t orennayar::GetSize()  {
+  return(sizeof(*this));
+}
+
 //
 //MicrofacetReflection
 //
@@ -498,6 +526,12 @@ point3f MicrofacetReflection::get_albedo(const ray& r_in, const hit_record& rec)
   return(albedo->value(rec.u, rec.v, rec.p));
 }
 
+size_t MicrofacetReflection::GetSize()  {
+  return(sizeof(*this));
+}
+
+
+
 //
 //MicrofacetTransmission
 //
@@ -584,6 +618,10 @@ point3f MicrofacetTransmission::SchlickFresnel(Float cosTheta) const {
   return pow5(1 - cosTheta) * (point3f(1.0f));
 }
 
+size_t MicrofacetTransmission::GetSize()  {
+  return(sizeof(*this));
+}
+
 //
 //Glossy
 //
@@ -614,7 +652,7 @@ point3f glossy::f(const ray& r_in, const hit_record& rec, const ray& scattered)
   vec3f wo = unit_vector(uvw.world_to_local(scattered.direction()));
   
   auto pow5 = [](Float v) { return (v * v) * (v * v) * v; };
-  point3f diffuse = (28.0f/(23.0f*M_PI)) * albedo->value(rec.u, rec.v, rec.p) *
+  point3f diffuse = (28.0f / (23.0f * M_PI)) * Rd * albedo->value(rec.u, rec.v,rec.p) *
     (point3f(1.0f) + -Rs) *
     (1.0 - pow5(1 - 0.5f * AbsCosTheta(wi))) *
     (1.0 - pow5(1 - 0.5f * AbsCosTheta(wo)));
@@ -623,14 +661,14 @@ point3f glossy::f(const ray& r_in, const hit_record& rec, const ray& scattered)
     return(vec3f(0.0f));
   }
   Float cosine  = dot(wh,wi);
-  if(cosine < 0) {
-    cosine = 0;
+  if(cosine < 0 || !SameHemisphere(wi,wo)) {
+    return(point3f(0));
   }
   point3f specular = distribution->D(wh, rec.u, rec.v) /
-    (4 * AbsDot(wi, wh) *
+      (4 * AbsDot(wi, wh) *
       std::fmax(AbsCosTheta(wi), AbsCosTheta(wo))) *
       SchlickFresnel(dot(wo, wh));
-  return((diffuse + specular) * cosine);
+  return((diffuse + specular) * cosine );
 }
 
 point3f glossy::SchlickFresnel(Float cosTheta) const {
@@ -642,6 +680,15 @@ point3f glossy::get_albedo(const ray& r_in, const hit_record& rec) const {
   return(albedo->value(rec.u, rec.v, rec.p));
 }
 
+
+size_t glossy::GetSize()  {
+  return(sizeof(*this));
+}
+
+//
+//Hair
+//
+
 std::array<Float, pMax + 1> hair::ComputeApPdf(Float cosThetaO, Float h) const {
   // Compute array of $A_p$ values for _cosThetaO_
   Float sinThetaO = SafeSqrt(1 - cosThetaO * cosThetaO);
@@ -885,6 +932,10 @@ point3f hair::f(const ray& r_in, const hit_record& rec, const ray& scattered) co
   return(fsum);
 }
 
+size_t hair::GetSize()  {
+  return(sizeof(*this));
+}
+
 // vec3f hair::SigmaAFromConcentration(Float ce, Float cp) {
 //   vec3f sigma_a;
 //   Float eumelaninSigmaA[3] = {0.419f, 0.697f, 1.37f};
