@@ -14,6 +14,7 @@ class plymesh : public hitable {
     plymesh() {}
    ~plymesh() {}
   plymesh(std::string inputfile, std::string basedir, std::shared_ptr<material> mat, 
+          std::shared_ptr<alpha_texture> alpha, std::shared_ptr<bump_texture> bump,
           Float scale, Float shutteropen, Float shutterclose, int bvh_type, random_gen rng,
           std::shared_ptr<Transform> ObjectToWorld, std::shared_ptr<Transform> WorldToObject, bool reverseOrientation);
   virtual bool hit(const ray& r, Float t_min, Float t_max, hit_record& rec, random_gen& rng);
@@ -23,8 +24,13 @@ class plymesh : public hitable {
   virtual std::string GetName() const {
     return(std::string("Plymesh"));
   }
+  size_t GetSize()  {
+    return(ply_mesh_bvh->GetSize() + sizeof(triangles));
+  }
+  std::pair<size_t,size_t> CountNodeLeaf();
+  
+  std::unique_ptr<TriangleMesh> mesh;
   std::shared_ptr<bvh_node> ply_mesh_bvh;
-  std::shared_ptr<material> mat_ptr;
   hitable_list triangles;
 };
 
